#include "mainWidget.h"



// List of Storage Devices
static const struct Storage Storage[] = {
    { "CF Card", "/dev/hda", "CFDeviceS" },
    { "SD Card", "/dev/mmcda", "SDDeviceS" },
//    { "CF Card", "/dev/sda", "CFDeviceS" },
//    { "SD Card", "/dev/sdb", "SDDeviceS" },
    { NULL, NULL, NULL },
};



//
mainWidget::mainWidget(QWidget* parent, const char* name, WFlags fl) : mainWidgetBase(parent, name, fl) {
    setWState(WState_Reserved1);

    config.load();

    if(config.getMode() == usbdConfig::storage) {
	RadioButton2->setChecked(true);
    }
    else {
	RadioButton1->setChecked(true);
    }
}



int mainWidget::loadPartitions() {
    QString cmd;
    QRegExp regexp;
    FILE *stream;
    char buf[256];
    regex_t reg_disk, reg_part;
    regmatch_t match[4];


    ListView1->clear();
    ListView1->setColumnAlignment(3, Qt::AlignRight);
    ListView1->setSorting(2, TRUE);

    // Create regular expression patterns
    regcomp(&reg_disk, REGEXP_DISK, REG_EXTENDED);
    regcomp(&reg_part, REGEXP_PART, REG_EXTENDED);


    // Continue whole all storages
    for(int sIndex = 0; Storage[sIndex].name != NULL; sIndex++) {
	cmd.sprintf(CMD_FDISK, Storage[sIndex].device);

	QListViewItem *item = NULL;
	QPixmap px = Resource::loadPixmap(Storage[sIndex].pixmap);

	// trying to get storage partitions    
	if((stream = popen(cmd.latin1(), "r")) != NULL) {

	    while(fgets(buf, sizeof(buf), stream)) {
		// Disk
		if(regexec(&reg_disk, buf, 4, match, 0) != REG_NOMATCH) {
    		    QString heads(QString((char *)&buf + match[1].rm_so).left(match[1].rm_eo - match[1].rm_so));
    		    QString sectors(QString((char *)&buf + match[2].rm_so).left(match[2].rm_eo - match[2].rm_so));
		    QString cyl(QString((char *)&buf + match[3].rm_so).left(match[3].rm_eo - match[3].rm_so));

		    double size = heads.toDouble() * sectors.toDouble() * cyl.toDouble() * 512;

		    item = new QListViewItem(ListView1, 0);
		    item->setOpen(TRUE);
		    item->setPixmap(0, px);
		    item->setText(1, QString(Storage[sIndex].name) + "   ");
		    item->setText(2, Storage[sIndex].device);
		    item->setText(3, heads.sprintf("%15.0f MB", size / 1048576));
		    
		    if(config.getDevice() == Storage[sIndex].device) {
			ListView1->setSelected(item, TRUE);
		    }
		}

		// Disk Partiions
		if((item != NULL) && (regexec(&reg_part, buf, 4, match, 0) != REG_NOMATCH)) {
    		    QString dev(QString((char *)&buf + match[1].rm_so).left(match[1].rm_eo - match[1].rm_so));
    		    QString size(QString((char *)&buf + match[2].rm_so).left(match[2].rm_eo - match[2].rm_so));
    		    QString fs_type(QString((char *)&buf + match[3].rm_so).left(match[3].rm_eo - match[3].rm_so));
		    fs_type.insert(0, "   ");

		    QListViewItem *item_2 = new QListViewItem(item, 0);
		    item_2->setText(2, dev);
		    item_2->setText(3, size.sprintf("%8.0f MB", size.toDouble() / 1024));
		    item_2->setText(4, fs_type);

		    if(config.getDevice() == dev) {
		    	ListView1->setSelected(item_2, TRUE);
		    }
		}
	    }
	}

	pclose(stream);
	
	if(item == NULL) {
	    item = new QListViewItem(ListView1, 0);
	    item->setPixmap(0, px);
	    item->setText(1, Storage[sIndex].name);
	    item->setText(2, Storage[sIndex].device);
	}

	item = NULL;
    }

    regfree(&reg_disk);
    regfree(&reg_part);
    
    if(ListView1->selectedItem() == NULL) {
	ListView1->setSelected(ListView1->firstChild(), TRUE);
    }
    
    return 0;
}



void mainWidget::accept() {
    SlDlgWait wait(this, tr("Please wait..."));
    wait.exec();

    config.save();
    config.restart();

    close();
}



void mainWidget::selectPCLink(int state) {
    if(state == 2) {
	PushButton1->setEnabled(true);
	config.setMode(usbdConfig::net);
    }
    else {
	PushButton1->setEnabled(false);
    }
}



void mainWidget::launchPCLink() {
    Global::execute("qtsamba");
}



void mainWidget::selectStorage(int state) {
    if(state == 2) {
	ListView1->setEnabled(true);
	loadPartitions();
	config.setMode(usbdConfig::storage);
    }
    else {
	ListView1->setEnabled(false);
    }
}



//
void mainWidget::selectDevice(QListViewItem *item) {
    config.setDevice(item->text(2));

    QString text;
    
    text = item->text(2);
}
